package com.bomostory.sceneeditmodule.utils

import android.media.MediaPlayer
import com.bomostory.sceneeditmodule.basicdata.Music
import com.bomostory.sceneeditmodule.basicdata.Scene
import com.bomostory.sceneeditmodule.basicdata.Story
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

class MoviePlayerV2 {
    interface OnMoviePlayListener {
        fun onMovieSceneUpdate(scene: Scene)
        fun onMoviePlayViewUpdate(x: Int)
        fun onMoviePlayTimeUpdate(time: Long)
        fun onMoviePlayComplete()
    }

    var moviePlayListener: OnMoviePlayListener? = null

    var story: Story? = null

    var period: Long = 0

    var timer: Timer? = null

    var currentTime: Long = 0

    var isPause = false

    var musicList: List<Music> = ArrayList()

    private var mediaPlayers = ArrayList<MediaPlayer>()

    private val recordPlayers = HashMap<Int, MediaPlayer>()


    fun init() {
        isPause = false

        timer?.cancel()
        timer?.purge()
        timer = null

        currentTime = 0

        updateMovie()

        tryStopRecord()

        tryStopMusic()

        initRecord()

        initMediaPlayer()

    }

    fun start() {
        if (timer == null) {
            timer = Timer()
            timer?.scheduleAtFixedRate(object : TimerTask() {
                override fun run() {
                    if (!isPause) {

                        currentTime++

                        updateMovie()

                        tryPlayRecord()

                        tryPlayMusic()

                        if (currentTime >= period) {
                            moviePlayListener?.onMoviePlayComplete()
                            init()
                        }
                    }
                }
            }, 0, 1)
        } else {
            isPause = false
        }
    }

    fun pause() {
        isPause = true

        tryPauseRecord()

        tryPauseMusic()
    }

    fun seekTo(currentTime: Long) {
        this.currentTime = currentTime

        updateMovie()

        trySeekRecord()

        trySeekMusic()
    }

    private fun updateMovie() {
        val scene = getScene(currentTime)
        scene?.apply {
            moviePlayListener?.onMovieSceneUpdate(this)
            record?.let {
                for (track in it.tracks) {
                    if (track.time == currentTime) {
                        moviePlayListener?.onMoviePlayViewUpdate(track.positionX)
                    }
                }
            }
        }
        moviePlayListener?.onMoviePlayTimeUpdate(currentTime)
    }

    private fun getScene(t: Long): Scene? {
        var currentScene: Scene? = null
        var startPeriod = 0L
        var endPeriod = 0L

        story?.apply {
            for (scene in scenes) {
                scene.record?.apply {
                    endPeriod += period
                    if (t in startPeriod..endPeriod) {
                        currentScene = scene
                        startPeriod = endPeriod
                    }
                }
            }
        }
        return currentScene
    }

    private fun initRecord() {
        recordPlayers.clear()
        story?.let { story ->
            for (scene in story.scenes) {
                scene.recordPath?.let {
                    val mediaPlayer = MediaPlayer()
                    mediaPlayer.apply {
                        reset()
                        setDataSource(scene.recordPath)
                        prepare()
                    }
                    recordPlayers.put(story.scenes.indexOf(scene), mediaPlayer)
                }
            }
        }
    }

    private fun initMediaPlayer() {
        mediaPlayers.clear()

        for (music in musicList) {
            val mediaPlayer = MediaPlayer()
            mediaPlayer.apply {
                reset()
                setDataSource(music.path)
                isLooping = music.isLoop
                prepare()
            }
            mediaPlayers.add(mediaPlayer)
        }
    }

    private fun tryPlayRecord() {
        story?.let {
            recordPlayers[it.scenes.indexOf(getScene(currentTime))]?.apply {
                if (!isPlaying) {
                    start()
                }
            }
        }
    }

    private fun tryPlayMusic() {
        for (music in musicList) {
            val musicEndTime = if (music.isLoop) (music.offsetTime + music.endTime - music.startTime) + music.loopDuration else (music.offsetTime + music.endTime - music.startTime)

            if (currentTime in music.offsetTime..musicEndTime) {
                if (!mediaPlayers[musicList.indexOf(music)].isPlaying) {
                    mediaPlayers[musicList.indexOf(music)].start()
                } else if (currentTime >= musicEndTime) {
                    mediaPlayers[musicList.indexOf(music)].stop()
                    mediaPlayers[musicList.indexOf(music)].prepare()
                }
            }
        }
    }

    private fun tryPauseRecord() {
        recordPlayers.forEach {
            it.value.pause()
        }
    }

    private fun tryPauseMusic() {
        for (mediaPlayer in mediaPlayers) {
            mediaPlayer.pause()
        }
    }

    private fun trySeekRecord() {
        story?.let {
            var period: Long = 0
            for (i in 0 until it.scenes.indexOf(getScene(currentTime))) {
                period += (it.scenes[i].record?.period ?: 0)
            }

            recordPlayers[it.scenes.indexOf(getScene(currentTime))]?.apply {
                if (period == 0L) {
                    this.seekTo(currentTime.toInt())
                } else {
                    this.seekTo((currentTime % period).toInt())
                }
            }

        }
    }

    private fun trySeekMusic() {
        for (music in musicList) {
            val musicEndTime = if (music.isLoop) (music.offsetTime + music.endTime - music.startTime) + music.loopDuration else (music.offsetTime + music.endTime - music.startTime)

            if (currentTime in music.offsetTime..musicEndTime) {
                val actualTime = if (currentTime - music.offsetTime >= music.endTime - music.startTime) ((currentTime - music.offsetTime).toDouble() / (music.endTime - music.startTime)).toInt()
                else (currentTime - music.offsetTime).toInt()
                mediaPlayers[musicList.indexOf(music)].seekTo(actualTime)
            } else {
                mediaPlayers[musicList.indexOf(music)].stop()
                mediaPlayers[musicList.indexOf(music)].prepare()
            }
        }
    }

    private fun tryStopRecord() {
        recordPlayers.forEach {
            it.value.stop()
        }
    }

    private fun tryStopMusic() {
        for (mediaPlayer in mediaPlayers) {
            mediaPlayer.stop()
        }
    }
}